% General Note: This is used for the theoretical prediction of the cache misses
% based on the paper: Y. Andreopoulos et. al. "High-level cache modeling for 2-D
% discrete wavelet transform implementations", Journal of VLSI Signal Processing,
% Kluwer Academic Publishers
% Copyright: Yiannis Andreopoulos - VUB Dept. ETRO/IMEC 
% email: yandreop@etro.vub.ac.be
% http://www.etro.vub.ac.be/Members/name_pages/ANDREOPOULOS.Yiannis/personal.htm

% Version 1.0

% Note #2: This material is provided "as is" without any warranty whatsoever. 
% Furthermore, any potential modifications made by the reader are subject to 
% his/hers judgement on suitability/applicabity/correctness. However, the author
% is interested for potential modifications or extensions created by third 
% parties and encourages e-mail communication in relation to this topic.
% Have fun!


clear;
TRUE=1;
FALSE=0;

% first determine what will be shown
show_theoretical = TRUE;
show_experimental = TRUE;
show_slope = FALSE;

% Important note: The experimental results contained herein are created with SimpleScalar v2.0
% (see www.simplescalar.org) and correspond to the case described in the paper, i.e. a 2-level
% cache, fully associative, with block-size 64 bytes for both levels and sizes:
% sL2=131072 and sD=8192.
% If you would like to verify the consistency of the model under other conditions, you should
% import the relevant measurements of your case of interest. Otherwise simply put
% FALSE in show_experimental in order not to plot the experimental measurements

% cache and transform parameters

% block sizes (bytes)
bL2=64;
bD=64;
% cache sizes (bytes)
conflict_factor=1;
% The sizes are affected by the conflict factor. Put 1 for fully associative.
sL2=131072*conflict_factor;
sD=8192*conflict_factor;
assoc=sD; % if this is equal to the sD then its fully associative
% number of lifting steps
S=1;

% M factor (filter size is assumed to be (2M+1)/(2M-1)
M=4;

% coefficient precision (bytes). 
% For S=1 put 2 (integers-to-integers), for S=2 put 4 (floating point)
coeff_prec=2;


% image sizes parameters. Only square images are treated.
init_size=128;
step_size=16;
fin_size=896;

% The initial and the final number of levels.
init_lev=4;
fin_lev=6;

% the step size of the experimental measurements is fixed (128).
% And also the initial size (128). So by knowing where the theoretical
% curves stop, we can select a specific part of the experimentals to use:
fin_exp=(fin_size)/128;
init_exp=(init_size)/128;

% the matrices that contain the theoretical points of each method
I_L2(1:fin_size/step_size-(init_size/step_size-1))=0;
I_D(1:fin_size/step_size-(init_size/step_size-1))=0;

II_L2(1:fin_size/step_size-(init_size/step_size-1))=0;
II_D(1:fin_size/step_size-(init_size/step_size-1))=0;

III_L2(1:fin_size/step_size-(init_size/step_size-1))=0;
III_D(1:fin_size/step_size-(init_size/step_size-1))=0;

tot_LWT_L2(1:fin_size/step_size-(init_size/step_size-1))=0;
tot_LWT_D(1:fin_size/step_size-(init_size/step_size-1))=0;

tot_LBWT_L2(1:fin_size/step_size-(init_size/step_size-1))=0;
tot_LBWT_D(1:fin_size/step_size-(init_size/step_size-1))=0;


% main loop, for every image size and every level, calc. the theoretical points for all the methods
for r=init_size:step_size:fin_size
   for lev=init_lev:1:fin_lev
      [RCWT_I_L2,RCWT_II_L2,RCWT_III_L2,RCWT_I_D,RCWT_II_D,RCWT_III_D]=rcwt_miss(r,r,lev,S,bL2,bD,coeff_prec,sL2,sD);
		[LWT_L2,LWT_D]=lwt_miss(r,r,lev,M,bL2,bD,coeff_prec,sL2,sD);
		[LBWT_L2,LBWT_D]=LBWT_miss(r,r,lev,M,bL2,bD,coeff_prec,sL2,sD);
      
      I_L2(r/step_size-(init_size/step_size-1))=I_L2(r/step_size-(init_size/step_size-1))+RCWT_I_L2;
      I_D(r/step_size-(init_size/step_size-1))=I_D(r/step_size-(init_size/step_size-1))+RCWT_I_D;
      
      II_L2(r/step_size-(init_size/step_size-1))=II_L2(r/step_size-(init_size/step_size-1))+RCWT_II_L2;
      II_D(r/step_size-(init_size/step_size-1))=II_D(r/step_size-(init_size/step_size-1))+RCWT_II_D;
      
      III_L2(r/step_size-(init_size/step_size-1))=III_L2(r/step_size-(init_size/step_size-1))+RCWT_III_L2;
      III_D(r/step_size-(init_size/step_size-1))=III_D(r/step_size-(init_size/step_size-1))+RCWT_III_D;
      
      tot_LWT_L2(r/step_size-(init_size/step_size-1))=tot_LWT_L2(r/step_size-(init_size/step_size-1))+LWT_L2;
      tot_LWT_D(r/step_size-(init_size/step_size-1))=tot_LWT_D(r/step_size-(init_size/step_size-1))+LWT_D;
      
      tot_LBWT_L2(r/step_size-(init_size/step_size-1))=tot_LBWT_L2(r/step_size-(init_size/step_size-1))+LBWT_L2;
      tot_LBWT_D(r/step_size-(init_size/step_size-1))=tot_LBWT_D(r/step_size-(init_size/step_size-1))+LBWT_D;
      
   end
end

im_size=[init_size:step_size:fin_size]
if show_slope == TRUE
   ss_theor = im_size(1:(fin_size-init_size)/step_size+1).^2;
else
   ss_theor = 1;
end



% experimentals
% these comes from the simplescalar simulator, so things may get messy here...

% In general the experimentals come from 128x128 256x256 384x384 512x512 640x640 896x896 1024x1024 images

% 9/7 filter

% format: exp_type_cache_type(level:4-6 typically , image size:128x128 - 1024x1024 typically)
exp_I_97_D=zeros(3,7);
exp_I_97_L2=zeros(3,7);
exp_II_97_D=zeros(3,7);
exp_II_97_L2=zeros(3,7);
exp_III_97_D=zeros(3,7);
exp_III_97_L2=zeros(3,7);

% case I
exp_I_97_D(1,1)=56076; exp_I_97_L2(1,1)=1471;
exp_I_97_D(2,1)=56076; exp_I_97_L2(2,1)=1471;
exp_I_97_D(3,1)=56076; exp_I_97_L2(3,1)=1471;

exp_I_97_D(1,2)=333699; exp_I_97_L2(1,2)=25658;
exp_I_97_D(2,2)=333699; exp_I_97_L2(2,2)=25658;
exp_I_97_D(3,2)=333699; exp_I_97_L2(3,2)=25658;

exp_I_97_D(1,3)=790026; exp_I_97_L2(1,3)=63896;
exp_I_97_D(2,3)=790051; exp_I_97_L2(2,3)=63896;
exp_I_97_D(3,3)=790051; exp_I_97_L2(3,3)=63896;

exp_I_97_D(1,4)=1486008; exp_I_97_L2(1,4)=115868;
exp_I_97_D(2,4)=1486073; exp_I_97_L2(2,4)=115868;
exp_I_97_D(3,4)=1486073; exp_I_97_L2(3,4)=115868;

exp_I_97_D(1,5)=2348440; exp_I_97_L2(1,5)=181319;
exp_I_97_D(2,5)=2348565; exp_I_97_L2(2,5)=181319;
exp_I_97_D(3,5)=2348565; exp_I_97_L2(3,5)=181319;

exp_I_97_D(1,6)=3413768; exp_I_97_L2(1,6)=266911;
exp_I_97_D(2,6)=3414293; exp_I_97_L2(2,6)=266911;
exp_I_97_D(3,6)=3414318; exp_I_97_L2(3,6)=266911;

exp_I_97_D(1,7)=4677029; exp_I_97_L2(1,7)=364667;
exp_I_97_D(2,7)=4677916; exp_I_97_L2(2,7)=364667;
exp_I_97_D(3,7)=4677973; exp_I_97_L2(3,7)=364667;

exp_I_97_D(1,8)=6181963; exp_I_97_L2(1,8)=477146;
exp_I_97_D(2,8)=6182986; exp_I_97_L2(2,8)=477146;
exp_I_97_D(3,8)=6183051; exp_I_97_L2(3,8)=477146;

% changed version which performs the two passes sequentially

exp_I_97_changed_D(1,1)=54522; exp_I_97_changed_L2(1,1)=1471;
exp_I_97_changed_D(2,1)=54522; exp_I_97_changed_L2(2,1)=1471;
exp_I_97_changed_D(3,1)=54522; exp_I_97_changed_L2(3,1)=1471;

exp_I_97_changed_D(1,2)=328028; exp_I_97_changed_L2(1,2)=17441;
exp_I_97_changed_D(2,2)=328028; exp_I_97_changed_L2(2,2)=17441;
exp_I_97_changed_D(3,2)=328028; exp_I_97_changed_L2(3,2)=17441;

exp_I_97_changed_D(1,3)=777008; exp_I_97_changed_L2(1,3)=40795;
exp_I_97_changed_D(2,3)=777033; exp_I_97_changed_L2(2,3)=40795;
exp_I_97_changed_D(3,3)=777033; exp_I_97_changed_L2(3,3)=40795;

exp_I_97_changed_D(1,4)=1463918; exp_I_97_changed_L2(1,4)=74826;
exp_I_97_changed_D(2,4)=1463983; exp_I_97_changed_L2(2,4)=74826;
exp_I_97_changed_D(3,4)=1463983; exp_I_97_changed_L2(3,4)=74826;

exp_I_97_changed_D(1,5)=2313950; exp_I_97_changed_L2(1,5)=117215;
exp_I_97_changed_D(2,5)=2314072; exp_I_97_changed_L2(2,5)=117215;
exp_I_97_changed_D(3,5)=2314072; exp_I_97_changed_L2(3,5)=117215;

exp_I_97_changed_D(1,6)=3364130; exp_I_97_changed_L2(1,6)=169996;
exp_I_97_changed_D(2,6)=3364361; exp_I_97_changed_L2(2,6)=169996;
exp_I_97_changed_D(3,6)=3364386; exp_I_97_changed_L2(3,6)=169996;

exp_I_97_changed_D(1,7)=4609488; exp_I_97_changed_L2(1,7)=232786;
exp_I_97_changed_D(2,7)=4609917; exp_I_97_changed_L2(2,7)=232786;
exp_I_97_changed_D(3,7)=4609974; exp_I_97_changed_L2(3,7)=232786;

exp_I_97_changed_D(1,8)=6094796; exp_I_97_changed_L2(1,8)=304920;
exp_I_97_changed_D(2,8)=6094796; exp_I_97_changed_L2(2,8)=304920;
exp_I_97_changed_D(3,8)=6094796; exp_I_97_changed_L2(3,8)=304920;


% 5/3 filter

exp_I_53_D(1,1)=35742; exp_I_53_L2(1,1)=939;
exp_I_53_D(2,1)=35742; exp_I_53_L2(2,1)=939;
exp_I_53_D(3,1)=35742; exp_I_53_L2(3,1)=939;

exp_I_53_D(1,2)=231704; exp_I_53_L2(1,2)=7136;
exp_I_53_D(2,2)=231704; exp_I_53_L2(2,2)=7136;
exp_I_53_D(3,2)=231704; exp_I_53_L2(3,2)=7136;

exp_I_53_D(1,3)=557412; exp_I_53_L2(1,3)=19746;
exp_I_53_D(2,3)=557412; exp_I_53_L2(2,3)=19746;
exp_I_53_D(3,3)=557412; exp_I_53_L2(3,3)=19746;

exp_I_53_D(1,4)=1059926; exp_I_53_L2(1,4)=35713;
exp_I_53_D(2,4)=1059958; exp_I_53_L2(2,4)=35713;
exp_I_53_D(3,4)=1059958; exp_I_53_L2(3,4)=35713;

exp_I_53_D(1,5)=1681806; exp_I_53_L2(1,5)=58240;
exp_I_53_D(2,5)=1681886; exp_I_53_L2(2,5)=58240;
exp_I_53_D(3,5)=1681886; exp_I_53_L2(3,5)=58240;

exp_I_53_D(1,6)=2452734; exp_I_53_L2(1,6)=84366;
exp_I_53_D(2,6)=2452830; exp_I_53_L2(2,6)=84366;
exp_I_53_D(3,6)=2452830; exp_I_53_L2(3,6)=84366;

exp_I_53_D(1,7)=3369917; exp_I_53_L2(1,7)=114969;
exp_I_53_D(2,7)=3370031; exp_I_53_L2(2,7)=114969;
exp_I_53_D(3,7)=3370031; exp_I_53_L2(3,7)=114969;

exp_I_53_D(1,8)=4457631; exp_I_53_L2(1,8)=150623;
exp_I_53_D(2,8)=4457835; exp_I_53_L2(2,8)=150623;
exp_I_53_D(3,8)=4457867; exp_I_53_L2(3,8)=150623;


% case II
exp_II_97_D(1,1)=26901; exp_II_97_L2(1,1)=1460;
exp_II_97_D(2,1)=26901; exp_II_97_L2(2,1)=1460;
exp_II_97_D(3,1)=26901; exp_II_97_L2(3,1)=1460;

exp_II_97_D(1,2)=186503; exp_II_97_L2(1,2)=17462;
exp_II_97_D(2,2)=186503; exp_II_97_L2(2,2)=17462;
exp_II_97_D(3,2)=186503; exp_II_97_L2(3,2)=17462;

exp_II_97_D(1,3)=445251; exp_II_97_L2(1,3)=40802;
exp_II_97_D(2,3)=445276; exp_II_97_L2(2,3)=40802;
exp_II_97_D(3,3)=445276; exp_II_97_L2(3,3)=40802;

exp_II_97_D(1,4)=824242; exp_II_97_L2(1,4)=74884;
exp_II_97_D(2,4)=824307; exp_II_97_L2(2,4)=74884;
exp_II_97_D(3,4)=824262; exp_II_97_L2(3,4)=74884;

exp_II_97_D(1,5)=1299438; exp_II_97_L2(1,5)=117288;
exp_II_97_D(2,5)=1299582; exp_II_97_L2(2,5)=117288;
exp_II_97_D(3,5)=1299583; exp_II_97_L2(3,5)=117288;

exp_II_97_D(1,6)=1879641; exp_II_97_L2(1,6)=170052;
exp_II_97_D(2,6)=1879871; exp_II_97_L2(2,6)=170052;
exp_II_97_D(3,6)=1879896; exp_II_97_L2(3,6)=170052;

exp_II_97_D(1,7)=2566456; exp_II_97_L2(1,7)=232890;
exp_II_97_D(2,7)=2566886; exp_II_97_L2(2,7)=232890;
exp_II_97_D(3,7)=2566942; exp_II_97_L2(3,7)=232889;

exp_II_97_D(1,8)=3483842; exp_II_97_L2(1,8)=304981;
exp_II_97_D(2,8)=3484344; exp_II_97_L2(2,8)=304981;
exp_II_97_D(3,8)=3484409; exp_II_97_L2(3,8)=304981;

exp_II_53_D(1,1)=46875; exp_II_53_L2(1,1)=1458;
exp_II_53_D(2,1)=46875; exp_II_53_L2(2,1)=1458;
exp_II_53_D(3,1)=46875; exp_II_53_L2(3,1)=1458;

exp_II_53_D(1,2)=196508; exp_II_53_L2(1,2)=15457;
exp_II_53_D(2,2)=196508; exp_II_53_L2(2,2)=15457;
exp_II_53_D(3,2)=196508; exp_II_53_L2(3,2)=15457;

exp_II_53_D(1,3)=442921; exp_II_53_L2(1,3)=36283;
exp_II_53_D(2,3)=442970; exp_II_53_L2(2,3)=36284;
exp_II_53_D(3,3)=442969; exp_II_53_L2(3,3)=36283;

exp_II_53_D(1,4)=794845; exp_II_53_L2(1,4)=66788;
exp_II_53_D(2,4)=794909; exp_II_53_L2(2,4)=66788;
exp_II_53_D(3,4)=794909; exp_II_53_L2(3,4)=66788;

exp_II_53_D(1,5)=1242155; exp_II_53_L2(1,5)=104606;
exp_II_53_D(2,5)=1242437; exp_II_53_L2(2,5)=104606;
exp_II_53_D(3,5)=1242476; exp_II_53_L2(3,5)=104606;

exp_II_53_D(1,6)=1788897; exp_II_53_L2(1,6)=151776;
exp_II_53_D(2,6)=1789250; exp_II_53_L2(2,6)=151776;
exp_II_53_D(3,6)=1789298; exp_II_53_L2(3,6)=151776;

exp_II_53_D(1,7)=2434800; exp_II_53_L2(1,7)=207998;
exp_II_53_D(2,7)=2435228; exp_II_53_L2(2,7)=207998;
exp_II_53_D(3,7)=2435284; exp_II_53_L2(3,7)=207998;

exp_II_53_D(1,8)=2612956; exp_II_53_L2(1,8)=225021;
exp_II_53_D(2,8)=2613431; exp_II_53_L2(2,8)=225021;
exp_II_53_D(3,8)=2613489; exp_II_53_L2(3,8)=225021;

exp_II_53_D(1,9)=2795226; exp_II_53_L2(1,9)=244715;
exp_II_53_D(2,9)=2799258; exp_II_53_L2(2,9)=244715;
exp_II_53_D(3,9)=2799318; exp_II_53_L2(3,9)=244715;

exp_II_53_D(1,10)=2985172; exp_II_53_L2(1,10)=1332721;
exp_II_53_D(2,10)=2993064; exp_II_53_L2(2,10)=1332721;
exp_II_53_D(3,10)=2993126; exp_II_53_L2(3,10)=1332721;

exp_II_53_D(1,11)=3083122; exp_II_53_L2(1,11)=2361350;
exp_II_53_D(2,11)=3091390; exp_II_53_L2(2,11)=2361350;
exp_II_53_D(3,11)=3091482; exp_II_53_L2(3,11)=2361350;

exp_II_53_D(1,12)=3133194; exp_II_53_L2(1,12)=2400478;
exp_II_53_D(2,12)=3141494; exp_II_53_L2(2,12)=2400478;
exp_II_53_D(3,12)=3141603; exp_II_53_L2(3,12)=2400478;

exp_II_53_D(1,13)=3180336; exp_II_53_L2(1,13)=2435134;
exp_II_53_D(2,13)=3188800; exp_II_53_L2(2,13)=2435134;
exp_II_53_D(3,13)=3188864; exp_II_53_L2(3,13)=2435134;


% case III
exp_III_97_D(1,1)=29700; exp_III_97_L2(1,1)=1467;
exp_III_97_D(2,1)=29700; exp_III_97_L2(2,1)=1467;
exp_III_97_D(3,1)=29700; exp_III_97_L2(3,1)=1467;

exp_III_97_D(1,2)=193935; exp_III_97_L2(1,2)=17521;
exp_III_97_D(2,2)=193935; exp_III_97_L2(2,2)=17521;
exp_III_97_D(3,2)=193935; exp_III_97_L2(3,2)=17521;

exp_III_97_D(1,3)=464198; exp_III_97_L2(1,3)=40923;
exp_III_97_D(2,3)=464223; exp_III_97_L2(2,3)=40923;
exp_III_97_D(3,3)=464223; exp_III_97_L2(3,3)=40923;

exp_III_97_D(1,4)=862470; exp_III_97_L2(1,4)=75005;
exp_III_97_D(2,4)=862535; exp_III_97_L2(2,4)=75005;
exp_III_97_D(3,4)=862535; exp_III_97_L2(3,4)=75005;

exp_III_97_D(1,5)=1402274; exp_III_97_L2(1,5)=117369;
exp_III_97_D(2,5)=1402441; exp_III_97_L2(2,5)=117369;
exp_III_97_D(3,5)=1402445; exp_III_97_L2(3,5)=117369;

exp_III_97_D(1,6)=2074434; exp_III_97_L2(1,6)=170175;
exp_III_97_D(2,6)=2074669; exp_III_97_L2(2,6)=170175;
exp_III_97_D(3,6)=2074694; exp_III_97_L2(3,6)=170175;

exp_III_97_D(1,7)=2853174; exp_III_97_L2(1,7)=233011;
exp_III_97_D(2,7)=2853612; exp_III_97_L2(2,7)=233011;
exp_III_97_D(3,7)=2853669; exp_III_97_L2(3,7)=233011;

exp_III_97_D(1,8)=3869231; exp_III_97_L2(1,8)=306614;
exp_III_97_D(2,8)=3869739; exp_III_97_L2(2,8)=306614;
exp_III_97_D(3,8)=3869804; exp_III_97_L2(3,8)=306614;

exp_III_53_D(1,1)=24166; exp_III_53_L2(1,1)=961;
exp_III_53_D(2,1)=24166; exp_III_53_L2(2,1)=961;
exp_III_53_D(3,1)=24166; exp_III_53_L2(3,1)=961;

exp_III_53_D(1,2)=175580; exp_III_53_L2(1,2)=7092;
exp_III_53_D(2,2)=175580; exp_III_53_L2(2,2)=7092;
exp_III_53_D(3,2)=175580; exp_III_53_L2(3,2)=7092;

exp_III_53_D(1,3)=421020; exp_III_53_L2(1,3)=19792;
exp_III_53_D(2,3)=421020; exp_III_53_L2(2,3)=19792;
exp_III_53_D(3,3)=421020; exp_III_53_L2(3,3)=19792;

exp_III_53_D(1,4)=781097; exp_III_53_L2(1,4)=35699;
exp_III_53_D(2,4)=781097; exp_III_53_L2(2,4)=35699;
exp_III_53_D(3,4)=781097; exp_III_53_L2(3,4)=35699;

exp_III_53_D(1,5)=1231741; exp_III_53_L2(1,5)=58319;
exp_III_53_D(2,5)=1231821; exp_III_53_L2(2,5)=58319;
exp_III_53_D(3,5)=1231821; exp_III_53_L2(3,5)=58319;

exp_III_53_D(1,6)=1782560; exp_III_53_L2(1,6)=84463;
exp_III_53_D(2,6)=1782657; exp_III_53_L2(2,6)=84463;
exp_III_53_D(3,6)=1782657; exp_III_53_L2(3,6)=84463;

exp_III_53_D(1,7)=2434707; exp_III_53_L2(1,7)=115078;
exp_III_53_D(2,7)=2434819; exp_III_53_L2(2,7)=115078;
exp_III_53_D(3,7)=2434819; exp_III_53_L2(3,7)=115078;

exp_III_53_D(1,8)=3200830; exp_III_53_L2(1,8)=150703;
exp_III_53_D(2,8)=3201033; exp_III_53_L2(2,8)=150703;
exp_III_53_D(3,8)=3201042; exp_III_53_L2(3,8)=150703;


% LWT
exp_LWT_97_D(1,1)=4023; exp_LWT_97_L2(1,1)=847;
exp_LWT_97_D(2,1)=3507; exp_LWT_97_L2(2,1)=879;
exp_LWT_97_D(3,1)=5021; exp_LWT_97_L2(3,1)=1003;

exp_LWT_97_D(1,2)=10988; exp_LWT_97_L2(1,2)=1047;
exp_LWT_97_D(2,2)=16755; exp_LWT_97_L2(2,2)=1207;
exp_LWT_97_D(3,2)=26324; exp_LWT_97_L2(3,2)=1610;

exp_LWT_97_D(1,3)=22708; exp_LWT_97_L2(1,3)=1214;
exp_LWT_97_D(2,3)=35788; exp_LWT_97_L2(2,3)=1410;
exp_LWT_97_D(3,3)=56922; exp_LWT_97_L2(3,3)=1884;

exp_LWT_97_D(1,4)=38596; exp_LWT_97_L2(1,4)=1384;
exp_LWT_97_D(2,4)=62223; exp_LWT_97_L2(2,4)=1617;
exp_LWT_97_D(3,4)=99301; exp_LWT_97_L2(3,4)=2159;

exp_LWT_97_D(1,5)=58949; exp_LWT_97_L2(1,5)=1550;
exp_LWT_97_D(2,5)=96019; exp_LWT_97_L2(2,5)=1820;
exp_LWT_97_D(3,5)=153337; exp_LWT_97_L2(3,5)=3095;

exp_LWT_97_D(1,6)=83340; exp_LWT_97_L2(1,6)=1717;
exp_LWT_97_D(2,6)=137136; exp_LWT_97_L2(2,6)=2023;
exp_LWT_97_D(3,6)=219098; exp_LWT_97_L2(3,6)=15074;

exp_LWT_97_D(1,7)=112050; exp_LWT_97_L2(1,7)=1884;
exp_LWT_97_D(2,7)=185692; exp_LWT_97_L2(2,7)=2237;
exp_LWT_97_D(3,7)=296593; exp_LWT_97_L2(3,7)=24337;

exp_LWT_97_D(1,8)=144947; exp_LWT_97_L2(1,8)=2053;
exp_LWT_97_D(2,8)=241586; exp_LWT_97_L2(2,8)=3006;
exp_LWT_97_D(3,8)=385839; exp_LWT_97_L2(3,8)=35604;

exp_LWT_53_D(1,1)=1366; exp_LWT_53_L2(1,1)=724;
exp_LWT_53_D(2,1)=1809; exp_LWT_53_L2(2,1)=743;
exp_LWT_53_D(3,1)=2350; exp_LWT_53_L2(3,1)=811;

exp_LWT_53_D(1,2)=6817; exp_LWT_53_L2(1,2)=821;
exp_LWT_53_D(2,2)=7807; exp_LWT_53_L2(2,2)=924;
exp_LWT_53_D(3,2)=14435; exp_LWT_53_L2(3,2)=1173;

exp_LWT_53_D(1,3)=16170; exp_LWT_53_L2(1,3)=826;
exp_LWT_53_D(2,3)=14525; exp_LWT_53_L2(2,3)=899;
exp_LWT_53_D(3,3)=16485; exp_LWT_53_L2(3,3)=1035;

exp_LWT_53_D(1,4)=24969; exp_LWT_53_L2(1,4)=961;
exp_LWT_53_D(2,4)=28449; exp_LWT_53_L2(2,4)=1146;
exp_LWT_53_D(3,4)=53067; exp_LWT_53_L2(3,4)=1525;

exp_LWT_53_D(1,5)=38245; exp_LWT_53_L2(1,5)=1038;
exp_LWT_53_D(2,5)=43777; exp_LWT_53_L2(2,5)=1257;
exp_LWT_53_D(3,5)=81508; exp_LWT_53_L2(3,5)=1700;

exp_LWT_53_D(1,6)=54294; exp_LWT_53_L2(1,6)=1116;
exp_LWT_53_D(2,6)=62334; exp_LWT_53_L2(2,6)=1369;
exp_LWT_53_D(3,6)=116027; exp_LWT_53_L2(3,6)=1877;

exp_LWT_53_D(1,7)=73236; exp_LWT_53_L2(1,7)=1193;
exp_LWT_53_D(2,7)=84256; exp_LWT_53_L2(2,7)=1479;
exp_LWT_53_D(3,7)=156607; exp_LWT_53_L2(3,7)=2051;

exp_LWT_53_D(1,8)=94907; exp_LWT_53_L2(1,8)=1271;
exp_LWT_53_D(2,8)=109499; exp_LWT_53_L2(2,8)=1591;
exp_LWT_53_D(3,8)=203339; exp_LWT_53_L2(3,8)=2241;

% LBWT
exp_LBWT_97_D(1,1)=6223; exp_LBWT_97_L2(1,1)=972;
exp_LBWT_97_D(2,1)=6834; exp_LBWT_97_L2(2,1)=1175;
exp_LBWT_97_D(3,1)=7197; exp_LBWT_97_L2(3,1)=1580;

exp_LBWT_97_D(1,2)=26947; exp_LBWT_97_L2(1,2)=1269;
exp_LBWT_97_D(2,2)=27249; exp_LBWT_97_L2(2,2)=1636;
exp_LBWT_97_D(3,2)=28086; exp_LBWT_97_L2(3,2)=2628;

exp_LBWT_97_D(1,3)=62142; exp_LBWT_97_L2(1,3)=1564;
exp_LBWT_97_D(2,3)=61025; exp_LBWT_97_L2(2,3)=2098;
exp_LBWT_97_D(3,3)=61570; exp_LBWT_97_L2(3,3)=12608;

exp_LBWT_97_D(1,4)=111799; exp_LBWT_97_L2(1,4)=1861;
exp_LBWT_97_D(2,4)=108494; exp_LBWT_97_L2(2,4)=3881;
exp_LBWT_97_D(3,4)=108013; exp_LBWT_97_L2(3,4)=46649;

exp_LBWT_97_D(1,5)=175993; exp_LBWT_97_L2(1,5)=2160;
exp_LBWT_97_D(2,5)=169143; exp_LBWT_97_L2(2,5)=34571;
exp_LBWT_97_D(3,5)=167182; exp_LBWT_97_L2(3,5)=87291;

exp_LBWT_97_D(1,6)=254301; exp_LBWT_97_L2(1,6)=2825;
exp_LBWT_97_D(2,6)=243142; exp_LBWT_97_L2(2,6)=57468;
exp_LBWT_97_D(3,6)=238842; exp_LBWT_97_L2(3,6)=135423;

exp_LBWT_97_D(1,7)=347660; exp_LBWT_97_L2(1,7)=82175;
exp_LBWT_97_D(2,7)=331160; exp_LBWT_97_L2(2,7)=120597;
exp_LBWT_97_D(3,7)=323258; exp_LBWT_97_L2(3,7)=195460;

exp_LBWT_97_D(1,8)=454000; exp_LBWT_97_L2(1,8)=117720;
exp_LBWT_97_D(2,8)=431783; exp_LBWT_97_L2(2,8)=221327;
exp_LBWT_97_D(3,8)=420233; exp_LBWT_97_L2(3,8)=276760;



exp_LBWT_53_D(1,1)=2454; exp_LBWT_53_L2(1,1)=776;
exp_LBWT_53_D(2,1)=3399; exp_LBWT_53_L2(2,1)=884;
exp_LBWT_53_D(3,1)=3864; exp_LBWT_53_L2(3,1)=1101;

exp_LBWT_53_D(1,2)=12690; exp_LBWT_53_L2(1,2)=919;
exp_LBWT_53_D(2,2)=13795; exp_LBWT_53_L2(2,2)=1124;
exp_LBWT_53_D(3,2)=14842; exp_LBWT_53_L2(3,2)=1534;

exp_LBWT_53_D(1,3)=26464; exp_LBWT_53_L2(1,3)=987;
exp_LBWT_53_D(2,3)=30059; exp_LBWT_53_L2(2,3)=1060;
exp_LBWT_53_D(3,3)=31501; exp_LBWT_53_L2(3,3)=1362;

exp_LBWT_53_D(1,4)=55971; exp_LBWT_53_L2(1,4)=1187;
exp_LBWT_53_D(2,4)=57694; exp_LBWT_53_L2(2,4)=1603;
exp_LBWT_53_D(3,4)=59476; exp_LBWT_53_L2(3,4)=2926;

exp_LBWT_53_D(1,5)=89089; exp_LBWT_53_L2(1,5)=1328;
exp_LBWT_53_D(2,5)=90962; exp_LBWT_53_L2(2,5)=1841;
exp_LBWT_53_D(3,5)=92870; exp_LBWT_53_L2(3,5)=16733;

exp_LBWT_53_D(1,6)=131022; exp_LBWT_53_L2(1,6)=1469;
exp_LBWT_53_D(2,6)=131429; exp_LBWT_53_L2(2,6)=2082;
exp_LBWT_53_D(3,6)=133548; exp_LBWT_53_L2(3,6)=36276;

exp_LBWT_53_D(1,7)=181180; exp_LBWT_53_L2(1,7)=1610;
exp_LBWT_53_D(2,7)=180797; exp_LBWT_53_L2(2,7)=2776;
exp_LBWT_53_D(3,7)=182212; exp_LBWT_53_L2(3,7)=63059;

exp_LBWT_53_D(1,8)=238963; exp_LBWT_53_L2(1,8)=1745;
exp_LBWT_53_D(2,8)=237349; exp_LBWT_53_L2(2,8)=34411;
exp_LBWT_53_D(3,8)=238165; exp_LBWT_53_L2(3,8)=107400;





% LBWT_VM7
exp_LBWT_VM7_97_D(1,1)=16346; exp_LBWT_VM7_97_L2(1,1)=1904;
exp_LBWT_VM7_97_D(2,1)=17699; exp_LBWT_VM7_97_L2(2,1)=2097;
exp_LBWT_VM7_97_D(3,1)=18489; exp_LBWT_VM7_97_L2(3,1)=2212;

exp_LBWT_VM7_97_D(1,2)=49395; exp_LBWT_VM7_97_L2(1,2)=2184;
exp_LBWT_VM7_97_D(2,2)=50788; exp_LBWT_VM7_97_L2(2,2)=2570;
exp_LBWT_VM7_97_D(3,2)=52962; exp_LBWT_VM7_97_L2(3,2)=2718;

exp_LBWT_VM7_97_D(1,3)=110357; exp_LBWT_VM7_97_L2(1,3)=2516;
exp_LBWT_VM7_97_D(2,3)=112485; exp_LBWT_VM7_97_L2(2,3)=2902;
exp_LBWT_VM7_97_D(3,3)=113548; exp_LBWT_VM7_97_L2(3,3)=3127;

exp_LBWT_VM7_97_D(1,4)=212193; exp_LBWT_VM7_97_L2(1,4)=2899;
exp_LBWT_VM7_97_D(2,4)=214345; exp_LBWT_VM7_97_L2(2,4)=3028;
exp_LBWT_VM7_97_D(3,4)=215640; exp_LBWT_VM7_97_L2(3,4)=3450;

exp_LBWT_VM7_97_D(1,5)=416967; exp_LBWT_VM7_97_L2(1,5)=3428;
exp_LBWT_VM7_97_D(2,5)=419462; exp_LBWT_VM7_97_L2(2,5)=3570;
exp_LBWT_VM7_97_D(3,5)=420026; exp_LBWT_VM7_97_L2(3,5)=3812;

exp_LBWT_VM7_97_D(1,6)=602531; exp_LBWT_VM7_97_L2(1,6)=3672;
exp_LBWT_VM7_97_D(2,6)=605316; exp_LBWT_VM7_97_L2(2,6)=3815;
exp_LBWT_VM7_97_D(3,6)=606401; exp_LBWT_VM7_97_L2(3,6)=4336;

exp_LBWT_VM7_97_D(1,7)=819609; exp_LBWT_VM7_97_L2(1,7)=4061;
exp_LBWT_VM7_97_D(2,7)=823031; exp_LBWT_VM7_97_L2(2,7)=4487;
exp_LBWT_VM7_97_D(3,7)=824422; exp_LBWT_VM7_97_L2(3,7)=4877;

exp_LBWT_VM7_97_D(1,8)=1125007; exp_LBWT_VM7_97_L2(1,8)=4671;
exp_LBWT_VM7_97_D(2,8)=1129096; exp_LBWT_VM7_97_L2(2,8)=4846;
exp_LBWT_VM7_97_D(3,8)=1130478; exp_LBWT_VM7_97_L2(3,8)=5204;

exp_LBWT_VM7_97_D_conv(1,1)=20590; exp_LBWT_VM7_97_L2_conv(1,1)=1961;
exp_LBWT_VM7_97_D_conv(2,1)=21661; exp_LBWT_VM7_97_L2_conv(2,1)=2142;
exp_LBWT_VM7_97_D_conv(3,1)=22309; exp_LBWT_VM7_97_L2_conv(3,1)=2242;

exp_LBWT_VM7_97_D_conv(1,2)=86217; exp_LBWT_VM7_97_L2_conv(1,2)=2695;
exp_LBWT_VM7_97_D_conv(2,2)=87143; exp_LBWT_VM7_97_L2_conv(2,2)=2914;
exp_LBWT_VM7_97_D_conv(3,2)=87823; exp_LBWT_VM7_97_L2_conv(3,2)=3008;

exp_LBWT_VM7_97_D_conv(1,3)=187341; exp_LBWT_VM7_97_L2_conv(1,3)=3084;
exp_LBWT_VM7_97_D_conv(2,3)=189351; exp_LBWT_VM7_97_L2_conv(2,3)=3344;
exp_LBWT_VM7_97_D_conv(3,3)=190471; exp_LBWT_VM7_97_L2_conv(3,3)=3559;

exp_LBWT_VM7_97_D_conv(1,4)=345046; exp_LBWT_VM7_97_L2_conv(1,4)=3496;
exp_LBWT_VM7_97_D_conv(2,4)=347079; exp_LBWT_VM7_97_L2_conv(2,4)=3691;
exp_LBWT_VM7_97_D_conv(3,4)=348313; exp_LBWT_VM7_97_L2_conv(3,4)=3935;

exp_LBWT_VM7_97_D_conv(1,5)=549020; exp_LBWT_VM7_97_L2_conv(1,5)=4261;
exp_LBWT_VM7_97_D_conv(2,5)=551815; exp_LBWT_VM7_97_L2_conv(2,5)=4499;
exp_LBWT_VM7_97_D_conv(3,5)=553269; exp_LBWT_VM7_97_L2_conv(3,5)=4730;

exp_LBWT_VM7_97_D_conv(1,6)=789684; exp_LBWT_VM7_97_L2_conv(1,6)=4678;
exp_LBWT_VM7_97_D_conv(2,6)=792911; exp_LBWT_VM7_97_L2_conv(2,6)=5004;
exp_LBWT_VM7_97_D_conv(3,6)=794540; exp_LBWT_VM7_97_L2_conv(3,6)=5274;

exp_LBWT_VM7_97_D_conv(1,7)=1084485; exp_LBWT_VM7_97_L2_conv(1,7)=6131;
exp_LBWT_VM7_97_D_conv(2,7)=1088524; exp_LBWT_VM7_97_L2_conv(2,7)=14933;
exp_LBWT_VM7_97_D_conv(3,7)=1090371; exp_LBWT_VM7_97_L2_conv(3,7)=21460;

exp_LBWT_VM7_97_D_conv(1,8)=1455535; exp_LBWT_VM7_97_L2_conv(1,8)=163664;
exp_LBWT_VM7_97_D_conv(2,8)=1460127; exp_LBWT_VM7_97_L2_conv(2,8)=172396;
exp_LBWT_VM7_97_D_conv(3,8)=1462046; exp_LBWT_VM7_97_L2_conv(3,8)=173789;



exp_LBWT_VM7_53_D(1,1)=13497; exp_LBWT_VM7_53_L2(1,1)=1857;
exp_LBWT_VM7_53_D(2,1)=14496; exp_LBWT_VM7_53_L2(2,1)=1992;
exp_LBWT_VM7_53_D(3,1)=15351; exp_LBWT_VM7_53_L2(3,1)=2124;

exp_LBWT_VM7_53_D(1,2)=40616; exp_LBWT_VM7_53_L2(1,2)=2068;
exp_LBWT_VM7_53_D(2,2)=41772; exp_LBWT_VM7_53_L2(2,2)=2197;
exp_LBWT_VM7_53_D(3,2)=42454; exp_LBWT_VM7_53_L2(3,2)=2450;

exp_LBWT_VM7_53_D(1,3)=88541; exp_LBWT_VM7_53_L2(1,3)=2309;
exp_LBWT_VM7_53_D(2,3)=90701; exp_LBWT_VM7_53_L2(2,3)=2601;
exp_LBWT_VM7_53_D(3,3)=91442; exp_LBWT_VM7_53_L2(3,3)=2924;

exp_LBWT_VM7_53_D(1,4)=162641; exp_LBWT_VM7_53_L2(1,4)=2595;
exp_LBWT_VM7_53_D(2,4)=164777; exp_LBWT_VM7_53_L2(2,4)=2912;
exp_LBWT_VM7_53_D(3,4)=165439; exp_LBWT_VM7_53_L2(3,4)=3027;

exp_LBWT_VM7_53_D(1,5)=288912; exp_LBWT_VM7_53_L2(1,5)=3090;
exp_LBWT_VM7_53_D(2,5)=290780; exp_LBWT_VM7_53_L2(2,5)=3338;
exp_LBWT_VM7_53_D(3,5)=291080; exp_LBWT_VM7_53_L2(3,5)=3547;

exp_LBWT_VM7_53_D(1,6)=427024; exp_LBWT_VM7_53_L2(1,6)=3455;
exp_LBWT_VM7_53_D(2,6)=429031; exp_LBWT_VM7_53_L2(2,6)=3558;
exp_LBWT_VM7_53_D(3,6)=429622; exp_LBWT_VM7_53_L2(3,6)=3772;

exp_LBWT_VM7_53_D(1,7)=590427; exp_LBWT_VM7_53_L2(1,7)=3788;
exp_LBWT_VM7_53_D(2,7)=593010; exp_LBWT_VM7_53_L2(2,7)=3886;
exp_LBWT_VM7_53_D(3,7)=593892; exp_LBWT_VM7_53_L2(3,7)=4190;

exp_LBWT_VM7_53_D(1,8)=825254; exp_LBWT_VM7_53_L2(1,8)=4079;
exp_LBWT_VM7_53_D(2,8)=828267; exp_LBWT_VM7_53_L2(2,8)=4396;
exp_LBWT_VM7_53_D(3,8)=829284; exp_LBWT_VM7_53_L2(3,8)=4806;


% find the average of all the levels
for i=1:1:fin_exp
   av_exp_I_97_L2(i)=mean(exp_I_97_changed_L2(:,i));
   av_exp_II_97_L2(i)=mean(exp_II_97_L2(:,i));
   av_exp_III_97_L2(i)=mean(exp_III_97_L2(:,i));
   
   av_exp_I_97_D(i)=mean(exp_I_97_changed_D(:,i));
   av_exp_II_97_D(i)=mean(exp_II_97_D(:,i));
   av_exp_III_97_D(i)=mean(exp_III_97_D(:,i));
   
   av_exp_LWT_97_L2(i)=mean(exp_LWT_97_L2(:,i));
   av_exp_LWT_97_D(i)=mean(exp_LWT_97_D(:,i));
   
   av_exp_LBWT_97_L2(i)=mean(exp_LBWT_97_L2(:,i));
   av_exp_LBWT_97_D(i)=mean(exp_LBWT_97_D(:,i));
   
   av_exp_LBWT_VM7_97_L2(i)=mean(exp_LBWT_VM7_97_L2(:,i));
   av_exp_LBWT_VM7_97_D(i)=mean(exp_LBWT_VM7_97_D(:,i));
   
   
   
   av_exp_I_53_L2(i)=mean(exp_I_53_L2(:,i));
   av_exp_II_53_L2(i)=mean(exp_II_53_L2(:,i));
   av_exp_III_53_L2(i)=mean(exp_III_53_L2(:,i));
   
   av_exp_I_53_D(i)=mean(exp_I_53_D(:,i));
   av_exp_II_53_D(i)=mean(exp_II_53_D(:,i));
   av_exp_III_53_D(i)=mean(exp_III_53_D(:,i));
   
   av_exp_LWT_53_L2(i)=mean(exp_LWT_53_L2(:,i));
   av_exp_LWT_53_D(i)=mean(exp_LWT_53_D(:,i));
   
   av_exp_LBWT_53_L2(i)=mean(exp_LBWT_53_L2(:,i));
   av_exp_LBWT_53_D(i)=mean(exp_LBWT_53_D(:,i));
   
   av_exp_LBWT_VM7_53_L2(i)=mean(exp_LBWT_VM7_53_L2(:,i));
   av_exp_LBWT_VM7_53_D(i)=mean(exp_LBWT_VM7_53_D(:,i));
   
end

% case II 5/3 has a special effect
for i=9:1:13
   av_exp_II_53_L2(i)=mean(exp_II_53_L2(:,i));
   av_exp_II_53_D(i)=mean(exp_II_53_D(:,i));
end





% find the scaling factors for the theoretical curves so that they match the experimental
if S==1 & coeff_prec==2
   % 5/3 filter
   %L2_exp_factor=1;
   L2_exp_factor=max(max([av_exp_I_53_L2(1:fin_exp) av_exp_II_53_L2(1:fin_exp) av_exp_III_53_L2(1:fin_exp)]));
   if fin_size==1024
      %L2_theor_factor=1;
      L2_theor_factor=(max(max([I_L2(1:49) II_L2(1:49) III_L2(1:49)]/(fin_lev-init_lev+1))));
   else
      %L2_theor_factor=L2_exp_factor/1.22;
      L2_theor_factor=(max(max([I_L2(1:(fin_size-init_size)/step_size+1) II_L2(1:(fin_size-init_size)/step_size+1) III_L2(1:(fin_size-init_size)/step_size+1)]/(fin_lev-init_lev+1))));
   end
   
   %D_exp_factor=1;
   D_exp_factor=max(max([av_exp_I_53_D(1:fin_exp) av_exp_II_53_D(1:fin_exp) av_exp_III_53_D(1:fin_exp)]));
   %D_theor_factor=D_exp_factor/1.22;
   D_theor_factor=(max(max([I_D(1:(fin_size-init_size)/step_size+1) II_D(1:(fin_size-init_size)/step_size+1) III_D(1:(fin_size-init_size)/step_size+1)]/(fin_lev-init_lev+1))));
   
   disp('L2 scaling factor:');
   L2_exp_factor/L2_theor_factor
   disp('D scaling factor:');
   D_exp_factor/D_theor_factor
end



if S==2 & coeff_prec==4
   % 9/7 filter
   %L2_exp_factor=1;
   L2_exp_factor=max(max([av_exp_I_97_L2(1:fin_exp) av_exp_II_97_L2(1:fin_exp) av_exp_III_97_L2(1:fin_exp)]));
   %L2_theor_factor=L2_exp_factor/1.22;
   L2_theor_factor=(max(max([I_L2(1:(fin_size-init_size)/step_size+1) II_L2(1:(fin_size-init_size)/step_size+1) III_L2(1:(fin_size-init_size)/step_size+1)]/(fin_lev-init_lev+1))));
   
   %D_exp_factor=1;
   D_exp_factor=max(max([av_exp_I_97_D(1:fin_exp) av_exp_II_97_D(1:fin_exp) av_exp_III_97_D(1:fin_exp)]));
   %D_theor_factor=D_exp_factor/1.22;
   D_theor_factor=(max(max([I_D(1:(fin_size-init_size)/step_size+1) II_D(1:(fin_size-init_size)/step_size+1) III_D(1:(fin_size-init_size)/step_size+1)]/(fin_lev-init_lev+1))));
   
   disp('L2 scaling factor:');
   L2_exp_factor/L2_theor_factor
   disp('D scaling factor:');
   D_exp_factor/D_theor_factor
end   

% if the slope values are shown (misses per pixel) then show only normalized values from 0~1
if show_slope == TRUE
%   L2_exp_factor = 1;
%   D_exp_factor = 1;
   L2_theor_factor = L2_theor_factor/fin_size^2;
   D_theor_factor = D_theor_factor/fin_size^2;
end


% plot the theoretical curves   
figure;

if show_theoretical == TRUE
      
   plot_RCWT_I_L2=plot(im_size(1:(fin_size-init_size)/step_size+1),(I_L2(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/L2_theor_factor./ss_theor,'k-');
	xlabel('image size (square)');
   if show_experimental == FALSE & show_slope == TRUE
      ylabel('misses per pixel (normalized)');
   else
      ylabel('misses: (# x 100) % of maximum');
   end
   
	hold;
	grid on;
	plot_RCWT_II_L2=plot(im_size(1:(fin_size-init_size)/step_size+1),(II_L2(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/L2_theor_factor./ss_theor,'k--'); %'b--');
	plot_RCWT_III_L2=plot(im_size(1:(fin_size-init_size)/step_size+1),(III_L2(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/L2_theor_factor./ss_theor,'k-.'); %'m-.');
	plot_tot_LWT_L2=plot(im_size(1:(fin_size-init_size)/step_size+1),(tot_LWT_L2(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/L2_theor_factor./ss_theor,'k:'); %'b:');
   plot_tot_LBWT_L2=plot(im_size(1:(fin_size-init_size)/step_size+1),(tot_LBWT_L2(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/L2_theor_factor./ss_theor,'k.-'); %'g.-');
   set(plot_RCWT_I_L2,'MarkerSize',8);
   set(plot_RCWT_I_L2,'LineWidth',2);
   set(plot_RCWT_II_L2,'MarkerSize',8);
   set(plot_RCWT_II_L2,'LineWidth',2);
   set(plot_RCWT_III_L2,'MarkerSize',8);
   set(plot_RCWT_III_L2,'LineWidth',2);
   set(plot_tot_LWT_L2,'MarkerSize',8);
   set(plot_tot_LWT_L2,'LineWidth',2);
   set(plot_tot_LBWT_L2,'MarkerSize',8);
   set(plot_tot_LBWT_L2,'LineWidth',2);
end

if S==1 & coeff_prec == 2
   title('L2-Cache misses (5/3 filter)');
end

if S==2 & coeff_prec == 4
   title('L2-Cache misses (9/7 filter)');
end

% plot the experimental curves for the specific filter-pair
points=[128 256 384 512 640 768 896 1024];
points_special=[128 256 384 512 640 768 896 928 960 992 1008 1016 1024];

if show_slope == TRUE
   ss_points=points.^2;
	ss_points_special=points_special.^2;   
else
   ss_points=1;
   ss_points_special=1;
end



if show_experimental == TRUE
   if S==1 & coeff_prec==2
   	% 5/3 filter
    
	   plot_exp_RCWT_I_L2=plot(points(init_exp:fin_exp),av_exp_I_53_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'ko'); %'ko');
   
	   % case II in the 5/3 filter has a special effect 
   	if fin_size==1024
      	plot_exp_RCWT_II_L2=plot(points_special,av_exp_II_53_L2/L2_exp_factor./ss_points_special,'k^'); %'b^');
	   else
   	   plot_exp_RCWT_II_L2=plot(points_special(init_exp:fin_exp),av_exp_II_53_L2(init_exp:fin_exp)/L2_exp_factor./ss_points_special,'k^'); %'b^');
	   end
      
	   plot_exp_RCWT_III_L2=plot(points(init_exp:fin_exp),av_exp_III_53_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'k+'); %'m+');
   
	   plot_exp_LWT_L2=plot(points(init_exp:fin_exp),av_exp_LWT_53_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'k*'); %'b*');
   
	   %plot_exp_LBWT_VM7_L2=plot(points(init_exp:fin_exp),av_exp_LBWT_VM7_53_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'ks'); %'gs');
   
	   plot_exp_LBWT_L2=plot(points(init_exp:fin_exp),av_exp_LBWT_53_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'kx'); %'gs');
	end

	if S==2 & coeff_prec==4
   	% 9/7 filter
   
	   plot_exp_RCWT_I_L2=plot(points(init_exp:fin_exp),av_exp_I_97_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'ko'); %'ko');
		plot_exp_RCWT_II_L2=plot(points(init_exp:fin_exp),av_exp_II_97_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'k^'); %'b^');
	   plot_exp_RCWT_III_L2=plot(points(init_exp:fin_exp),av_exp_III_97_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'k+'); %'m+');
   
	   plot_exp_LWT_L2=plot(points(init_exp:fin_exp),av_exp_LWT_97_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'k*'); %'b*');
   
	   %plot_exp_LBWT_VM7_L2=plot(points(init_exp:fin_exp),av_exp_LBWT_VM7_97_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'ks'); %'gs');
   
   	plot_exp_LBWT_L2=plot(points(init_exp:fin_exp),av_exp_LBWT_97_L2(init_exp:fin_exp)/L2_exp_factor./ss_points,'kx'); %'gs');
   end
   set(plot_exp_RCWT_I_L2,'MarkerSize',8);
   set(plot_exp_RCWT_I_L2,'LineWidth',2);
   set(plot_exp_RCWT_II_L2,'MarkerSize',8);
   set(plot_exp_RCWT_II_L2,'LineWidth',2);
   set(plot_exp_RCWT_III_L2,'MarkerSize',8);
   set(plot_exp_RCWT_III_L2,'LineWidth',2);
   set(plot_exp_LWT_L2,'MarkerSize',8);
   set(plot_exp_LWT_L2,'LineWidth',2);
   set(plot_exp_LBWT_L2,'MarkerSize',8);
   set(plot_exp_LBWT_L2,'LineWidth',2);
end


if show_theoretical == TRUE & show_experimental == FALSE
   legend([plot_RCWT_I_L2 plot_RCWT_II_L2 plot_RCWT_III_L2 plot_tot_LWT_L2 plot_tot_LBWT_L2],'RCWT case I','RCWT case II','RCWT case III','LWT & LBWT (expected)','LWT & LBWT (streaming)',2);
end   

if show_theoretical == FALSE & show_experimental == TRUE
   legend([plot_exp_RCWT_I_L2 plot_exp_RCWT_II_L2 plot_exp_RCWT_III_L2 plot_exp_LWT_L2 plot_exp_LBWT_L2],'SSv2.0 RCWT case I','SSv2.0 RCWT case II','SSv2.0 RCWT case III','SSv2.0 LWT','SSv2.0 LBWT',2);
end   
   
if show_theoretical == TRUE & show_experimental == TRUE
   legend([plot_RCWT_I_L2 plot_RCWT_II_L2 plot_RCWT_III_L2 plot_tot_LWT_L2 plot_tot_LBWT_L2 plot_exp_RCWT_I_L2 plot_exp_RCWT_II_L2 plot_exp_RCWT_III_L2 plot_exp_LWT_L2 plot_exp_LBWT_L2],'RCWT case I','RCWT case II','RCWT case III','LWT & LBWT (expected)','LWT & LBWT (streaming)','SSv2.0 RCWT case I','SSv2.0 RCWT case II','SSv2.0 RCWT case III','SSv2.0 LWT','SSv2.0 LBWT',2);
end




% again, plot the theoretical and the experimental curves, for the D-Cache this time
figure;

if show_theoretical == TRUE
   plot_RCWT_I_D=plot(im_size(1:(fin_size-init_size)/step_size+1),(I_D(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/D_theor_factor./ss_theor,'k-'); %'k-');
	xlabel('image size (square)');
   if show_experimental == FALSE & show_slope == TRUE
      ylabel('misses per pixel (normalized)');
   else
      ylabel('misses: (# x 100) % of maximum');
   end
	hold;
	grid on;
	plot_RCWT_II_D=plot(im_size(1:(fin_size-init_size)/step_size+1),(II_D(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/D_theor_factor./ss_theor,'k--'); %'b--');
	plot_RCWT_III_D=plot(im_size(1:(fin_size-init_size)/step_size+1),(III_D(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/D_theor_factor./ss_theor,'k-.'); %'m-.');

	plot_tot_LWT_D=plot(im_size(1:(fin_size-init_size)/step_size+1),(tot_LWT_D(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/D_theor_factor./ss_theor,'k:'); %'b:');

	plot_tot_LBWT_D=plot(im_size(1:(fin_size-init_size)/step_size+1),(tot_LBWT_D(1:(fin_size-init_size)/step_size+1)/(fin_lev-init_lev+1))/D_theor_factor./ss_theor,'k.-'); %'g.-');
   set(plot_RCWT_I_D,'MarkerSize',8);
   set(plot_RCWT_I_D,'LineWidth',2);
   set(plot_RCWT_II_D,'MarkerSize',8);
   set(plot_RCWT_II_D,'LineWidth',2);
   set(plot_RCWT_III_D,'MarkerSize',8);
   set(plot_RCWT_III_D,'LineWidth',2);
   set(plot_tot_LWT_D,'MarkerSize',8);
   set(plot_tot_LWT_D,'LineWidth',2);
   set(plot_tot_LBWT_D,'MarkerSize',8);
   set(plot_tot_LBWT_D,'LineWidth',2);
   
end

if S==1 & coeff_prec == 2
   title('D-Cache misses (5/3 filter)');
end

if S==2 & coeff_prec == 4
   title('D-Cache misses (9/7 filter)');
end



if show_experimental == TRUE
	if S==1 & coeff_prec==2
   	% 5/3 filter

		plot_exp_RCWT_I_D=plot(points(init_exp:fin_exp),av_exp_I_53_D(init_exp:fin_exp)/D_exp_factor./ss_points,'ko'); %'ko');
	   % case II in the 5/3 has a special effect (only in the L2 cache however)
	   if fin_size==1024
   	   plot_exp_RCWT_II_D=plot(points_special,av_exp_II_53_D/D_exp_factor./ss_points_special,'k^'); %'b^');
	   else
		   plot_exp_RCWT_II_D=plot(points_special(init_exp:fin_exp),av_exp_II_53_D(init_exp:fin_exp)/D_exp_factor./ss_points_special,'k^'); %'b^');
	   end
   
		plot_exp_RCWT_III_D=plot(points(init_exp:fin_exp),av_exp_III_53_D(init_exp:fin_exp)/D_exp_factor./ss_points,'k+'); %'m+');

		plot_exp_LWT_D=plot(points(init_exp:fin_exp),av_exp_LWT_53_D(init_exp:fin_exp)/D_exp_factor./ss_points,'k*'); %'b*');
   
   	%plot_exp_LBWT_VM7_D=plot(points(init_exp:fin_exp),av_exp_LBWT_VM7_53_D(init_exp:fin_exp)/D_exp_factor./ss_points,'ks'); %'gs');
   
	   plot_exp_LBWT_D=plot(points(init_exp:fin_exp),av_exp_LBWT_53_D(init_exp:fin_exp)/D_exp_factor./ss_points,'kx'); %'gs');
   
	end

	if S==2 & coeff_prec==4
   	% 9/7 filter


		plot_exp_RCWT_I_D=plot(points(init_exp:fin_exp),av_exp_I_97_D(init_exp:fin_exp)/D_exp_factor./ss_points,'ko'); %'ko');
		plot_exp_RCWT_II_D=plot(points(init_exp:fin_exp),av_exp_II_97_D(init_exp:fin_exp)/D_exp_factor./ss_points,'k^'); %'b^');
		plot_exp_RCWT_III_D=plot(points(init_exp:fin_exp),av_exp_III_97_D(init_exp:fin_exp)/D_exp_factor./ss_points,'k+'); %'m+');

		plot_exp_LWT_D=plot(points(init_exp:fin_exp),av_exp_LWT_97_D(init_exp:fin_exp)/D_exp_factor./ss_points,'k*'); %'b*');
	
		%plot_exp_LBWT_VM7_D=plot(points(init_exp:fin_exp),av_exp_LBWT_VM7_97_D(init_exp:fin_exp)/D_exp_factor./ss_points,'ks'); %'gs');
   
		plot_exp_LBWT_D=plot(points(init_exp:fin_exp),av_exp_LBWT_97_D(init_exp:fin_exp)/D_exp_factor./ss_points,'kx'); %'gs');
   end
   set(plot_exp_RCWT_I_D,'MarkerSize',8);
   set(plot_exp_RCWT_I_D,'LineWidth',2);
   set(plot_exp_RCWT_II_D,'MarkerSize',8);
   set(plot_exp_RCWT_II_D,'LineWidth',2);
   set(plot_exp_RCWT_III_D,'MarkerSize',8);
   set(plot_exp_RCWT_III_D,'LineWidth',2);
   set(plot_exp_LWT_D,'MarkerSize',8);
   set(plot_exp_LWT_D,'LineWidth',2);
   set(plot_exp_LBWT_D,'MarkerSize',8);
   set(plot_exp_LBWT_D,'LineWidth',2);
   
end
   

if show_theoretical == TRUE & show_experimental == FALSE
   legend([plot_RCWT_I_D plot_RCWT_II_D plot_RCWT_III_D plot_tot_LWT_D plot_tot_LBWT_D],'RCWT case I','RCWT case II','RCWT case III','LWT','LBWT',2);
end   

if show_theoretical == FALSE & show_experimental == TRUE
   legend([plot_exp_RCWT_I_D plot_exp_RCWT_II_D plot_exp_RCWT_III_D plot_exp_LWT_D plot_exp_LBWT_D],'SSv2.0 RCWT case I','SSv2.0 RCWT case II','SSv2.0 RCWT case III','SSv2.0 LWT','SSv2.0 LBWT',2);
end   
   
if show_theoretical == TRUE & show_experimental == TRUE
   legend([plot_RCWT_I_D plot_RCWT_II_D plot_RCWT_III_D plot_tot_LWT_D plot_tot_LBWT_D plot_exp_RCWT_I_D plot_exp_RCWT_II_D plot_exp_RCWT_III_D plot_exp_LWT_D plot_exp_LBWT_D],'RCWT case I','RCWT case II','RCWT case III','LWT','LBWT','SSv2.0 RCWT case I','SSv2.0 RCWT case II','SSv2.0 RCWT case III','SSv2.0 LWT','SSv2.0 LBWT',2);
end
   





