function [RCWT_I_L2,RCWT_II_L2,RCWT_III_L2,RCWT_I_D,RCWT_II_D,RCWT_III_D] = rcwt_miss(N1,N2,LEVELS,S,bL2,bD,cP, sL2, sD)
% calculates the L2-Cache and D-Cache misses of the Row-Column Wavelet Transform
% cases I, II, III
% syntax:
% rcwt_miss(rows,cols,levels,lift_steps,blocksize_L2,blocksize_D,cP,size_L2,size_D)

% initialize everything, add the memory allocation misses:
if S==1
   fS=2;
else
   fS=1;
end

mem_alloc_L2=fS*N1*N2*cP/bL2;
mem_alloc_D=fS*N1*N2*cP/bD;

L2_miss=0; D_miss=0; 
RCWT_I_row_L2=mem_alloc_L2; RCWT_II_row_L2=mem_alloc_L2; RCWT_III_row_L2=mem_alloc_L2;
RCWT_I_row_D=mem_alloc_D; RCWT_II_row_D=mem_alloc_D;  RCWT_III_row_D=mem_alloc_D; 
RCWT_I_col_L2=0; RCWT_II_col_L2=0; RCWT_III_col_L2=0;
RCWT_I_col_D=0; RCWT_II_col_D=0;  RCWT_III_col_D=0;
   
for l=0:1:LEVELS-1
   % the L2-Cache column misses when skipping from row to column filtering are complex 
   % thus they are calculated separately here and this calculation is valid for all cases 
   R2C_miss_I=0;   
   a=0;
   if N1*N2*2^(-2*l)*cP>sL2   
         % this happens only when the entire row-filt. res. doesn't fit in the L2-Cache.
      
	   Ml_I=(N1*N2*2^(-2*l)*cP-sL2)/(N2*2^(-l)*cP);

      a(1)=Ml_I*bL2/(N2*2^(-l)*cP-bL2);
      for i=1: 1: N2*2^(-l)*cP/bL2
         if i>1
           	a(i)=(Ml_I*bL2+sum(a(1:i-1)))/(N2*2^(-l)*cP-bL2);
         end
         R2C_miss_I = R2C_miss_I + min([(Ml_I+sum(a)) (N1*2^(-l))]);
      end
   end
   
   R2C_miss_II=0;   
   a=0;
   if S==1
      if 2*N1*N2*2^(-2*l)*cP>sL2   	
         % this happens only when the entire row-filt. res. doesn't fit in the L2-Cache.
         Ml_II=(N1*N2*2^(-2*l)*cP-sL2/2)/(N2*2^(-l)*cP);

	      a(1)=Ml_II*bL2/(N2*2^(-l)*cP-bL2);
   	   for i=1: 1: N2*2^(-l)*cP/bL2
      	   if i>1
         	  	a(i)=(Ml_II*bL2+sum(a(1:i-1)))/(N2*2^(-l)*cP-bL2);
	         end
   	      R2C_miss_II = R2C_miss_II + min([(Ml_II+sum(a)) (N1*2^(-l))]);
      	end
      end
   else
      R2C_miss_II = R2C_miss_I;
   end
   
   
   % case I row misses
   if N1*N2*2^(-2*(l-1))*cP<sL2
      fL2=0;
   else
      fL2=1;
   end
   RCWT_I_row_L2 = RCWT_I_row_L2 + N1*N2*cP*2^(-2*l)/bL2*fL2;
   RCWT_I_row_D = RCWT_I_row_D + N1*N2*cP*2^(-2*l)/bD;
   
   % case I column misses
  	RCWT_I_col_L2 = RCWT_I_col_L2 + R2C_miss_I;
  
   temp=N1*2^(-l);
   if temp*bD<sD
      fD=0;
   else
      if temp*bD/2<sD
         fD=1;
      else
         fD=1.5;
      end
   end
   
   if temp*bD>sD & S > 1
      fS=S;
   else
      fS=1;
   end
   RCWT_I_col_D = RCWT_I_col_D + temp*N2*2^(-l)*(fS+fD+cP/(2*bD));
   
   % case II row misses
   if S==1;
      fS=2;
   else
      fS=1;
   end
   
   if fS*N1*N2*2^(-2*(l-1))*cP<sL2
      fL2=0;
   else
      fL2=1;
   end
   RCWT_II_row_L2 = RCWT_II_row_L2 + fS*N1*N2*cP*2^(-2*l)/bL2*fL2;
   
   RCWT_II_row_D = RCWT_II_row_D + (temp/bD)*fS;
   
   % case II column misses
   if S==1
      gS=1.5;
   else
      gS=1;
   end
   
   if sD<gS*(N1)*2^(-l)*bD-(fS-2)*N1*2^(-l)*cP/bD
      fD=1;
   else
      fD=0;
   end
      
   RCWT_II_col_L2 = RCWT_II_col_L2 + fS*R2C_miss_II; %(temp/bL2)*fL2*fS;
   RCWT_II_col_D = RCWT_II_col_D + N1*N2*(2^(-2*l))*(1+fD);
   
   % case III row misses
   if N1*N2*2^(-2*(l-1))*cP<sL2
      fL2=0;
   else
      fL2=1;
   end
   RCWT_III_row_L2 = RCWT_III_row_L2 + N1*N2*cP*2^(-2*l)/bL2*fL2;
   
   RCWT_III_row_D=RCWT_I_row_D;
   
   %case III column misses
   temp=(N1*N2*2^(-2*l))*cP;
   if temp<sL2
      fL2=0;
   else
      fL2=1;
   end
   
   RCWT_III_col_L2 = RCWT_III_col_L2 + R2C_miss_I; %(temp/bL2)*fL2;
   
   
   if S==1
      fS=1.5;
   else
      fS=2;
   end
   
   temp=N1*2^(-l);
   if temp*bD+fS*temp*cP/bD<sD
      fD=0;
   else
      fD=1;
   end
   
   RCWT_III_col_D = RCWT_III_col_D + temp*N2*2^(-l)*(1+fD+(cP*fS)/bD);
   
end

RCWT_I_L2=RCWT_I_row_L2+RCWT_I_col_L2;
RCWT_II_L2=RCWT_II_row_L2+RCWT_II_col_L2;
RCWT_III_L2=RCWT_III_row_L2+RCWT_III_col_L2;

RCWT_I_D=RCWT_I_row_D+RCWT_I_col_D;
RCWT_II_D=RCWT_II_row_D+RCWT_II_col_D;
RCWT_III_D=RCWT_III_row_D+RCWT_III_col_D;
